import smbus, crc, time, gpio, spidev
from datetime import datetime

MAX31875_ADDRESS = 0x4A
VEML3328_ADDRESS = 0x10
ATTACH_ADC_RELAY_GPIO = 4

# ADC Reference (VCC)
VCC=3.317

# Delay between measurements
DELAY = 15 * 60

max11410_spi = spidev.SpiDev()
max11410_spi.open(0, 0)
max11410_spi.max_speed_hz = 1000000
max11410_spi.mode = 0

def max11410_write_reg8(addr, val):
	max11410_spi.xfer([addr, val])

def max11410_write_reg16(addr, val):
	max11410_spi.xfer([addr, val >> 8, val & 0xFF])

def max11410_read_reg24(addr):
	received_data = max11410_spi.xfer([addr | 0x80, 0, 0, 0])
	return received_data[3] | (received_data[2] << 8) | (received_data[1] << 16)

def max11410_read_reg24_bytes(addr):
	received_data = max11410_spi.xfer([addr | 0x80, 0, 0, 0])
	return [received_data[3] ,received_data[2], received_data[1]]

def max11410_wait_for_conversion_complete():
	while True:
		if max11410_read_reg24(0x38) & 0x10:
			return
		
def max11410_measure_voltage():
	max11410_write_reg8(0x01, 0x00) # start single conversion
	max11410_wait_for_conversion_complete()
	value = int.from_bytes(max11410_read_reg24_bytes(0x30), "little", signed="True")
	return (VCC * value / 2**23) * 2.58 / 1.557

def max11410_init():
	# reset sensor
	max11410_write_reg8(0x00, 0x03)

	#configure sensor
	max11410_write_reg8(0x08, 0x3F) # select filter:    FILTER=SINC4 RATE=1920sps
	max11410_write_reg8(0x09, 0x03) # select reference: P=AVCC N=AGND
	max11410_write_reg8(0x0B, 0x9A) # select input:     P=AIN9 N=AGND

i2c_bus = smbus.SMBus(1)

def max31875_init():
	# One Shot: No (0b0)
	# Conversion Rate: 1sps (0b01)
	# PEC: Enabled (0b1)
	# Timeout: Enabled (0b0)
	# Resolution: 12bit (0b11)
	# Data format: Extended (0b1)
	# Shutdown: Disabled (0b0)
	# Comp/Int: do not matter (0b0)
	# Fault queue: 1 (0b00)
	# Config value: 0x00EA
	i2c_bus.write_i2c_block_data(MAX31875_ADDRESS, 0x01, [0x00, 0xEA, 0xFD])

def max31875_get_temperature():
	buffer = i2c_bus.read_i2c_block_data(MAX31875_ADDRESS, 0x00, 3)
	crc_params = crc.Configuration(
		    width=8,
		    polynomial=0x07,
		    init_value=0x00,
		    final_xor_value=0x00,
		    reverse_input=False,
		    reverse_output=False,
		)
	crc_buffer = [MAX31875_ADDRESS << 1, 0x00, (MAX31875_ADDRESS << 1) | 0x01, buffer[0], buffer[1]]
	crc_expected = crc.Calculator(crc_params, True).checksum(bytes(crc_buffer))
	if buffer[2] != crc_expected:
		raise Exception("CRC Error. Comptued: " + str(crc_expected) + " Received: " + str(buffer[2]))

	temperature_reg = (buffer[0] << 8) | buffer[1]
	temperature_val = ((temperature_reg & 0x7FFF) >> 3) * 0.0625
	if temperature_reg & 0x8000:
		temperature_val = -temperature_val

	return temperature_val

def veml3328_init():
	# SD0: Power on (0b0)
	# TRIG: No trigger (0b0)
	# AF: Auto (0b0)
	# IT: 100ms (0b01)
	# SENS: High (0b0)
	# GAIN: 1 (0b00)
	# DG: 1 (0b00)
	# SD_ALS only: all ch (0b0)
	# SD1: power on (0b0)
	# Config value: 0x0010
	i2c_bus.write_i2c_block_data(VEML3328_ADDRESS, 0x00, [0x10, 0x00])

def veml3328_get_c():
	buffer = i2c_bus.read_i2c_block_data(VEML3328_ADDRESS, 0x04, 2)
	return buffer[0] | ((buffer[1]) << 8)

def relay_init():
	gpio.setup(ATTACH_ADC_RELAY_GPIO, gpio.OUT)
	gpio.output(ATTACH_ADC_RELAY_GPIO, gpio.LOW)

def relay_attach_adc():
	gpio.output(ATTACH_ADC_RELAY_GPIO, gpio.HIGH)
	time.sleep(1)

def relay_dettach_adc():
	gpio.output(ATTACH_ADC_RELAY_GPIO, gpio.LOW)

max31875_init()
veml3328_init()
relay_init()
max11410_init()

print("Starting", flush=True)

while True:
	time_str = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
	temperature = max31875_get_temperature()
	light = veml3328_get_c()

	relay_attach_adc()
	voltage = 0
	samples = 10
	for i in range(samples):
		measurement = max11410_measure_voltage()
		voltage += measurement
	voltage = voltage / samples
	relay_dettach_adc()

	print(time_str + "\tTemperature " + str(temperature) + "\tLight " + str(light) + "\tVoltage " + str(voltage), flush=True)
	time.sleep(DELAY)
	